function [AdaAMResult, W, feature_proj] = AdaAM( fea, gnd, c, k, W, rep)
%
%   fea: num*dim data matrix, each row is a data point
%   gnd: ground truth for testing
%   c  : number of the clusters
%   k  : number of k-NN neighbors for heat kernel
%   W : affinity matrix (default [])
%   rep: repeatation of the final k-Means clustering


rand('seed',0);
folder_now = pwd;  
addpath('./LPP');

[num, origDim] = size(fea);

if nargin < 3
    c = max(gnd);
end

if nargin < 4
    k = round(log2(num/c));
end

if nargin < 5
     W = [];
end

if nargin < 6
    rep = 100;
end

fea = bsxfun(@minus, fea, mean(fea,1));

options = [];
options.ReducedDim = c;
options.NeighborMode = 'KNN';
options.k = k;
options.WeightMode = 'HeatKernel';

if isempty(W)
    W = constructW(fea,options);
end

D = sparse(1:num,1:num,sum(W, 2));
L = D - W;
L = max(L, L');

%%%%%%%%%%%%%%%%%%%%%%%%%
%% AdaAM
%%%%%%%%%%%%%%%%%%%%%%%%%

if origDim < 350
    dimSelected = origDim/10;
else
    dimSelected = origDim/35;
end

[eigvec, eigval] = mySVD(fea);
P = eigvec(:,1:round(dimSelected));
P= P./repmat(sqrt(sum(P.^2,2)),1,size(P,2));
Delta = P*P';


%% sparsity

sorted = sort(-Delta(:));
thre = -sorted(round(numel(Delta)*(1/c/2.5)));
clear sorted
Delta(abs(Delta)<thre)=0;


XLDelX = fea'*(L-Delta)*fea;
XLDelX = max(XLDelX, XLDelX');
[eigvec, eigval] = eig(XLDelX);
eignz = eigvec(:,abs(diag(eigval))>1e-5);
A = eignz(:,1:round(dimSelected*5));


[eigvec, eigval] = mySVD(fea*A);
P = eigvec(:,1:round(dimSelected));
P= P./repmat(sqrt(sum(P.^2,2)),1,size(P,2));
Delta = P*P';


%% sparsity

sorted = sort(-Delta(:));
thre = -sorted(round(numel(Delta)*(1/c/5)));
clear sorted
Delta(abs(Delta)<thre)=0;


%% LPP

W = sparse(Delta-diag(sum(Delta))+D);
[eigvector, eigvalue] = LPP(W,options,fea);
feature_proj = real(eigvector);

%% k-Means

AdaAMResult = zeros(rep,3);
for cnt = 1:rep
    label=litekmeans(fea*feature_proj,c,'MaxIter',100,'Replicates',10);
    result = ClusteringMeasure(gnd, label);
    AdaAMResult(cnt,:) = result;
end

disp('AdaAM')
disp('    Accuraty:')
disp(['        Avg:', num2str(mean(AdaAMResult(:,1)))])
disp(['        Max:', num2str(max(AdaAMResult(:,1)))])
disp(['        Min:', num2str(min(AdaAMResult(:,1)))])
disp(['        Std:', num2str(std(AdaAMResult(:,1)))])

% 
% KNNResult = zeros(rep,3);
% [eigvector, eigvalue] = LPP(W,options,fea);


% for cnt = 1:rep
%    label=litekmeans(fea*eigvector,c,'MaxIter',100,'Replicates',10);
%    result = ClusteringMeasure(gnd, label);
%    KNNResult(cnt,:) = result;
% end


% disp('KNN')
% disp('    Accuraty:')
% disp(['        Avg:', num2str(mean(KNNResult(:,1)))])
% disp(['        Max:', num2str(max(KNNResult(:,1)))])
% disp(['        Min:', num2str(min(KNNResult(:,1)))])
% disp(['        Std:', num2str(std(KNNResult(:,1)))])


rmpath('./LPP');
